@@ -307,7 +307,7 @@ RGDAL_GDAL_DATA_Info(void) {
 
     PROTECT(ans=NEW_CHARACTER(1));
     installErrorHandler();
-    SET_STRING_ELT(ans, 0, COPY_TO_USER_STRING(CSVFilename( "prime_meridian.csv" )));
+    SET_STRING_ELT(ans, 0, COPY_TO_USER_STRING(CSVFilename( "stateplane.csv" )));
     uninstallErrorHandlerAndTriggerError();
 
     UNPROTECT(1);
@@ -319,37 +319,47 @@ SEXP
 RGDAL_GDALwithGEOS(void) {
     SEXP ans;
 
+//    int withGEOS;
+
+#if GDAL_VERSION_MAJOR >= 3 // New GDAL
+
+    PROTECT(ans=NEW_CHARACTER(1));
+    SET_STRING_ELT(ans, 0, COPY_TO_USER_STRING(GDALVersionInfo("BUILD_INFO")));
+
+#else // Old GDAL
+
     PROTECT(ans=NEW_LOGICAL(1));
 
     CPLPushErrorHandler(CPLQuietErrorHandler);
     saved_err_no = 0;
 
-    int withGEOS;
     OGRGeometry *poGeometry1, *poGeometry2;
     char* pszWKT;
     pszWKT = (char*) "POINT (10 20)";
 #if GDAL_VERSION_MAJOR == 1 || ( GDAL_VERSION_MAJOR == 2 && GDAL_VERSION_MINOR <= 2 ) // thanks to Even Roualt https://github.com/OSGeo/gdal/issues/681
 //#if GDAL_VERSION_MAJOR <= 2 && GDAL_VERSION_MINOR <= 2
     OGRGeometryFactory::createFromWkt( &pszWKT, NULL, &poGeometry1 );
-#else
+#else // l 339
     OGRGeometryFactory::createFromWkt( (const char*) pszWKT, NULL, &poGeometry1 );
-#endif
+#endif // l 339
     pszWKT = (char*) "POINT (30 20)";
 #if GDAL_VERSION_MAJOR == 1 || ( GDAL_VERSION_MAJOR == 2 && GDAL_VERSION_MINOR <= 2 ) // thanks to Even Roualt https://github.com/OSGeo/gdal/issues/681
 //#if GDAL_VERSION_MAJOR <= 2 && GDAL_VERSION_MINOR <= 2
     OGRGeometryFactory::createFromWkt( &pszWKT, NULL, &poGeometry2 );
-#else
+#else // l 346
     OGRGeometryFactory::createFromWkt( (const char*) pszWKT, NULL, &poGeometry2 );
-#endif
-    withGEOS = 1;
+#endif // l 346
+    int withGEOS = 1;
     if (poGeometry1->Union(poGeometry2) == NULL) withGEOS = 0;//FIXME VG
     OGRGeometryFactory::destroyGeometry(poGeometry1);
     OGRGeometryFactory::destroyGeometry(poGeometry2);
 
     CPLPopErrorHandler();
     saved_err_no = 0;
-
     LOGICAL_POINTER(ans)[0] = withGEOS;
+
+#endif // New GDAL
+
     UNPROTECT(1);
 
     return(ans);
@@ -844,11 +854,19 @@ RGDAL_GetRasterCount(SEXP sDataset) {
 
 /* changed to return proj4 string 20060212 RSB */
 SEXP
-RGDAL_GetProjectionRef(SEXP sDataset) {
+RGDAL_GetProjectionRef(SEXP sDataset, SEXP enforce_xy) {
 
-  OGRSpatialReference oSRS, oSRS1;
+  OGRSpatialReference *oSRS = new OGRSpatialReference;
   char *pszSRS_WKT = NULL;
-  SEXP ans;
+  SEXP ans, Datum, ToWGS84, Ellps;
+  int i, pc=0;
+  const char *datum, *towgs84, *ellps;
+/*  int vis_order;
+
+  if (enforce_xy == R_NilValue) vis_order = 0;
+  else if (LOGICAL_POINTER(enforce_xy)[0] == 1) vis_order = 1;
+  else if (LOGICAL_POINTER(enforce_xy)[0] == 0) vis_order = 0;
+  else vis_order = 0;*/
 
   GDALDataset *pDataset = getGDALDatasetPtr(sDataset);
   
@@ -856,23 +874,94 @@ RGDAL_GetProjectionRef(SEXP sDataset) {
   pszSRS_WKT = (char*) pDataset->GetProjectionRef();
   uninstallErrorHandlerAndTriggerError();
 
+  PROTECT(ans = NEW_CHARACTER(1)); pc++;
+
+  if (strlen(pszSRS_WKT) == 0) {
+    SET_STRING_ELT(ans, 0, COPY_TO_USER_STRING(""));
+    UNPROTECT(pc); 
+    return(ans);
+  }
+
   installErrorHandler();
 #if GDAL_VERSION_MAJOR == 1 || ( GDAL_VERSION_MAJOR == 2 && GDAL_VERSION_MINOR <= 2 ) // https://github.com/OSGeo/gdal/issues/681
 //#if GDAL_VERSION_MAJOR <= 2 && GDAL_VERSION_MINOR <= 2 
-  oSRS.importFromWkt( &pszSRS_WKT );
+  oSRS->importFromWkt( &pszSRS_WKT );
 #else
-  oSRS.importFromWkt( (const char*) pszSRS_WKT );
+  oSRS->importFromWkt( (const char*) pszSRS_WKT );
 #endif
-  oSRS.exportToProj4( &pszSRS_WKT );
-
   uninstallErrorHandlerAndTriggerError();
-  PROTECT(ans = NEW_CHARACTER(1));
-  SET_STRING_ELT(ans, 0, COPY_TO_USER_STRING(pszSRS_WKT));
 
-  installErrorHandler();
-  CPLFree( pszSRS_WKT );
-  uninstallErrorHandlerAndTriggerError();
-  UNPROTECT(1);
+  if (oSRS != NULL) {
+
+    installErrorHandler();
+    datum = oSRS->GetAttrValue("DATUM");
+    uninstallErrorHandlerAndTriggerError();
+    PROTECT(Datum = NEW_CHARACTER(1)); pc++;
+    if (datum != NULL) SET_STRING_ELT(Datum, 0, COPY_TO_USER_STRING(datum));
+
+    installErrorHandler();
+    ellps = oSRS->GetAttrValue("DATUM|SPHEROID");
+    uninstallErrorHandlerAndTriggerError();
+    PROTECT(Ellps = NEW_CHARACTER(1)); pc++;
+    if (ellps != NULL) SET_STRING_ELT(Ellps, 0, COPY_TO_USER_STRING(ellps));
+
+    PROTECT(ToWGS84 = NEW_CHARACTER(7)); pc++;
+    installErrorHandler();
+    for (i=0; i<7; i++) {
+      towgs84 = oSRS->GetAttrValue("TOWGS84", i);
+      if (towgs84 != NULL) SET_STRING_ELT(ToWGS84, i,
+        COPY_TO_USER_STRING(towgs84));
+    }
+    uninstallErrorHandlerAndTriggerError();
+
+#if GDAL_VERSION_MAJOR >= 3
+    installErrorHandler();
+        oSRS->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
+    uninstallErrorHandlerAndTriggerError();
+
+    SEXP WKT2_2018;
+    char *wkt2=NULL;
+    char **papszOptions = NULL;
+
+    PROTECT(WKT2_2018 = NEW_CHARACTER(1)); pc++;
+
+    installErrorHandler();
+    papszOptions = CSLAddString(papszOptions, "FORMAT=WKT2_2018");
+    papszOptions = CSLAddString(papszOptions, "MULTILINE=YES");
+    uninstallErrorHandlerAndTriggerError();
+
+    installErrorHandler();
+    if (oSRS->exportToWkt(&wkt2, papszOptions) != OGRERR_NONE) {
+      SET_STRING_ELT(WKT2_2018, 0, NA_STRING);
+    } else {
+      SET_STRING_ELT(WKT2_2018, 0, COPY_TO_USER_STRING(wkt2));
+      CPLFree( wkt2 );
+    }
+    uninstallErrorHandlerAndTriggerError();
+    setAttrib(ans, install("WKT2_2018"), WKT2_2018);
+#endif
+  }
+
+  if (oSRS != NULL) {
+    installErrorHandler();
+    if (oSRS->exportToProj4( &pszSRS_WKT ) != OGRERR_NONE) {
+      SET_STRING_ELT(ans, 0, COPY_TO_USER_STRING(""));
+    } else {
+      SET_STRING_ELT(ans, 0, COPY_TO_USER_STRING(pszSRS_WKT));
+      CPLFree( pszSRS_WKT );
+    }
+    uninstallErrorHandlerAndTriggerError();
+  } else SET_STRING_ELT(ans, 0, COPY_TO_USER_STRING(""));
+
+
+  if (oSRS != NULL) {
+    setAttrib(ans, install("towgs84"), ToWGS84);
+    setAttrib(ans, install("datum"), Datum);
+    setAttrib(ans, install("ellps"), Ellps);
+  }
+  delete oSRS;
+
+  UNPROTECT(pc);
   return(ans);
 
 }
@@ -1481,8 +1570,9 @@ RGDAL_GetRasterData(SEXP sxpRasterBand,
   // Create matrix transposed
   int pc=0;
   SEXP sRStorage;
-  PROTECT(sRStorage = allocMatrix(uRType,
-			       INTEGER(sxpDimOut)[1],
+// Mathias Moser https://stat.ethz.ch/pipermail/r-sig-geo/2020-April/028072.html
+  PROTECT(sRStorage = allocVector(uRType,
+			       ((R_xlen_t) INTEGER(sxpDimOut)[1]) *
 			       INTEGER(sxpDimOut)[0])); pc++;
 
 // replication for 2.4.0 RSB 20060726
@@ -1571,7 +1661,7 @@ RGDAL_GetRasterData(SEXP sxpRasterBand,
     noDataValue = NUMERIC_POINTER(NDV)[0];
   }
 
-  int i;
+  /*int*/ R_xlen_t i;
 
   if (hasNoDataValue) {
 
@@ -1579,7 +1669,7 @@ RGDAL_GetRasterData(SEXP sxpRasterBand,
 
     case INTSXP:
 
-      for (i = 0; i < LENGTH(sRStorage); ++i)
+      for (i = 0; i < XLENGTH(sRStorage); ++i)
 	if (INTEGER(sRStorage)[i] == (int) noDataValue) {
 	  INTEGER(sRStorage)[i] = NA_INTEGER;
 	}
@@ -1592,7 +1682,7 @@ RGDAL_GetRasterData(SEXP sxpRasterBand,
 
         case GDT_Float32:
 
-        for (i = 0; i < LENGTH(sRStorage); ++i)
+        for (i = 0; i < XLENGTH(sRStorage); ++i)
 	  if (REAL(sRStorage)[i] == (double) ((float) noDataValue)) {
 	    REAL(sRStorage)[i] = NA_REAL;
 	  }
@@ -1600,7 +1690,7 @@ RGDAL_GetRasterData(SEXP sxpRasterBand,
 
         case GDT_Float64:
 
-        for (i = 0; i < LENGTH(sRStorage); ++i)
+        for (i = 0; i < XLENGTH(sRStorage); ++i)
 	  if (REAL(sRStorage)[i] == (double) (noDataValue)) {
 	    REAL(sRStorage)[i] = NA_REAL;
 	  }
@@ -1627,7 +1717,7 @@ RGDAL_GetRasterData(SEXP sxpRasterBand,
   } else {
     installErrorHandler();
     if (uRType == REALSXP && pRasterBand->GetRasterDataType() == GDT_Float32) {
-        for (i = 0; i < LENGTH(sRStorage); ++i)
+        for (i = 0; i < XLENGTH(sRStorage); ++i)
 	  if (ISNAN(REAL(sRStorage)[i])) {
 	    REAL(sRStorage)[i] = NA_REAL;
 	  }
@@ -1870,14 +1960,23 @@ RGDAL_SetGeoTransform(SEXP sxpDataset, SEXP GeoTransform) {
 SEXP
 RGDAL_SetProject(SEXP sxpDataset, SEXP proj4string) {
 
-  OGRSpatialReference oSRS;
+  OGRSpatialReference *oSRS = new OGRSpatialReference;
   char *pszSRS_WKT = NULL;
 
   GDALDataset *pDataset = getGDALDatasetPtr(sxpDataset);
 
   installErrorHandler();
-  oSRS.importFromProj4(CHAR(STRING_ELT(proj4string, 0)));
-  oSRS.exportToWkt( &pszSRS_WKT );
+  oSRS->importFromProj4(CHAR(STRING_ELT(proj4string, 0)));
+  uninstallErrorHandlerAndTriggerError();
+
+#if GDAL_VERSION_MAJOR >= 3
+  installErrorHandler();
+    oSRS->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
+  uninstallErrorHandlerAndTriggerError();
+#endif
+  
+  installErrorHandler();
+  oSRS->exportToWkt( &pszSRS_WKT );
   uninstallErrorHandlerAndTriggerError();
 
   installErrorHandler();
@@ -1886,10 +1985,53 @@ RGDAL_SetProject(SEXP sxpDataset, SEXP proj4string) {
 
   if (err == CE_Failure) 
 	warning("Failed to set projection\n");
+        delete oSRS;
   uninstallErrorHandlerAndTriggerError();
 
   return(sxpDataset);
 }
+/* added RSB 20191103 */
+SEXP
+RGDAL_SetProject_WKT2(SEXP sxpDataset, SEXP WKT2string, SEXP enforce_xy) {
+
+#if GDAL_VERSION_MAJOR >= 3
+
+  OGRSpatialReference *oSRS = new OGRSpatialReference;
+  int vis_order;
+
+  if (enforce_xy == R_NilValue) vis_order = 0;
+  else if (LOGICAL_POINTER(enforce_xy)[0] == 1) vis_order = 1;
+  else if (LOGICAL_POINTER(enforce_xy)[0] == 0) vis_order = 0;
+  else vis_order = 0;
+
+  GDALDataset *pDataset = getGDALDatasetPtr(sxpDataset);
+
+//Rprintf("%s\n", CHAR(STRING_ELT(WKT2string, 0)));
+
+  installErrorHandler();
+  oSRS->importFromWkt(CHAR(STRING_ELT(WKT2string, 0)));
+  uninstallErrorHandlerAndTriggerError();
+
+  installErrorHandler();
+  if (vis_order == 1)
+    oSRS->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
+  uninstallErrorHandlerAndTriggerError();
+
+  installErrorHandler();
+  OGRErr err = pDataset->SetSpatialRef(oSRS);
+
+  if (err == CE_Failure) {
+	warning("Failed to set projection\n");
+        delete oSRS;
+  }
+  uninstallErrorHandlerAndTriggerError();
+
+  return(sxpDataset);
+#else
+  return(R_NilValue);
+#endif
+}
+
 
 SEXP RGDAL_SetRasterColorTable(SEXP raster, SEXP icT, SEXP ricT, SEXP cicT) {
 
